function [shape_space] = train_shape_space_model(options)
% Build a shape space model from images. Takes as input only an options structure and returns a model structure. This function can be called with the same options by multiple jobs to do certain parts in parallel.
% Assumptions:
% Images are the same size and are square in XY.
% 
% Options you must specify for good results:
% number_images, the number of images used to train this model
% image_function, a function that takes an index from 1 to options.number_images and returns the corresponding image
% convergence_absolute_error, the sum of the absolute differences between any two images at which interpolation convergence will be assumed. Specifically, during an interpolation between a source and a target image, the images will be iteratively deformed towards each other. After each iteration, this convergence test will be done.
% voxel_size
% shape_aspect_ratio, the 
% save_location
% 
% Options you should specify for completeness:
% maximum_shape_space_dimension, the maximum dimensionality for the shape space to be constructed (maximum value of 7)
% explained_variance_threshold, the minimum amount of variance that must be explained by the automatically selected dimensionality of the shape space, but this can only produce a dimensionality up to maximum_shape_space_dimension (between 0 and 1)
% 
% Returns a structure containing:
% positions, the positions of each shape in the constructed shape space
% convex_hull, the convex hull of those positions
% tessellation, the Delaunay tessellation of those positions
% 
% Other options can be seen in the default_options structure at the beginning of this function.
% 
% Dependencies from File Exchange:
% DataHash
% Dependencies from the Murphy and Rohde labs:
% 
% History:
% 2013-02-13 tebuck: Copied from test_walk_ternary025.m.
% 
% To do:
% Use MDS approximations.

% 9/22/13 grj - improved readability


  % We require certain options:
  required_options = {'image_function', 'convergence_absolute_error', 'voxel_size', 'shape_aspect_ratio', 'save_location', 'number_images'};
  for index = 1:length(required_options)
    required_option = required_options{index};
    if ~isfield(options, required_option)
      error('Option %s is required', required_option)
    end
  end


  first_image = options.image_function(1);

  [M, N, P] = size(first_image);
  [X_0, Y_0, Z_0] = meshgrid(1:N, 1:M, 1:P);
  identity_map = {X_0, Y_0, Z_0}; 
  clear X_0 Y_0 Z_0
  
  window_size = [M, N, P];




  
  % Default options go in a structure passed to process_options_structure:
  default_options = struct();
  
  
  % Parameters for windowed LDDMM (these should probably be in a substructure so field names do not overlap (not that they should as of 2013-02-14)):
  default_options.filter_radius = 32;
  % By default use windows the size of the images:
  default_options.window_radius = window_size(1);
  % Flat cells:
  default_options.kernel_z_radius = default_options.filter_radius * options.shape_aspect_ratio(3);
  % Flat cells:
  default_options.maximum_deformation_per_step = [1, 1, .5];

  default_options.maximum_iterations = 1000;
  
  default_options.just_compute_distance = true;

  % Options specific to this function:
  default_options.maximum_shape_space_dimension = 7;
  % The proportion of the variance that must be explained by the shape space is used to choose the dimensionality (up to options.maximum_shape_space_dimension):
  default_options.explained_variance_threshold = .9;
  % If less than (options.number_images - 3), some distances will not be computed and instead reconstructed from the rest of the matrix:
  default_options.desired_shape_space_dimensionality = inf;
  
  default_options.useCurrentResults = false;
  
  
  % Combine default and given options:
  options = process_options_structure(default_options, options); 
  
  
  base_filename = [options.save_location, filesep];

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % parallel: Computing pairwise distances between images:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  distances_filename = [base_filename, 'distances'];
  comparison_filename_prefix = distances_filename;
  % comparison_filename_prefix = base_filename;
  [distances, success] = compute_distance_matrix(options.image_function, options.number_images, comparison_filename_prefix, options, @(s, t)0, options.desired_shape_space_dimensionality, options.useCurrentResults);
%   if ~success
%     error('Cannot continue, some unfinished distances are being computed elsewhere.')
%   end


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % serial: Compute shape space:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Limit to 7 dimensions based on rule of thumb at <http://www.qhull.org/html/qh-code.htm>:
  maximum_shape_space_dimension = min(options.maximum_shape_space_dimension, 7);
  explained_variance_threshold = options.explained_variance_threshold;

  shape_space_filename = [base_filename, 'shape_space'];
  [result, success] = compute_shape_space(distances, 0, [explained_variance_threshold, maximum_shape_space_dimension], shape_space_filename);
  if ~success
    error('Cannot continue, shape space being computed elsewhere.')
  end

  shape_space =                         struct();
  shape_space.positions =               result.y2;
  shape_space.convex_hull =             result.convex_hull;
  shape_space.tessellation =            result.tes;
  shape_space.explained_variances =     result.explained_variances;
  shape_space.distances =               distances;
  shape_space.shape_space_options =     rmfield(options, {'image_function'});

end
