function [segimg, top_slice, bot_slice, sumslices,zflip] = preprocess(img, imgFile, psfPath, downsample, adherent, top_thresh, bot_thresh, display, cellmask)
%PREPROCESS segments and preprocesses a given image.
%
%Inputs:
% img = image array to be processed
% imgFile = mat file containing image to be processed
% psfPath = path to psf to use
% downsample = [1xn] or single number defining the amount to downsample the image before preprocessing.
% adherent = boolean flag defining if the cell is adherent(forces largest slice to bottom)
% top_thresh = the fraction of fluorescence you wish to consider as the top slice
% bot_thresh = the fraction of fluorescence you wish to consider as the bottom slice
% display = boolean flag of whether to display progress on screen
% cellmask = binary valued image array masking the inside of the cell
%
%Outputs:
% segimg = segmented image
% top_slice = integer value of the top slice where signal lies
% bot_slice = integer value of the top slice where signal lies
% sumslices = the cumsum of each slice
% zflip = boolean flag indicating if preprocess thinks the cell is upsidedown

% Copyright (C) 2006  Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%Created by Devin Sullivan 3/21/13 moved from being a subfunction of
%seg_cell_and_nucleus.m
% 
%G. Johnson 8/22/13 - support for 2D masks, even if img is 3D
%G. Johnson 8/26/13 - Re-mask the output of region_seg incase the snake moves
%                     out of bounds

if ~exist('cellmask', 'var') | isempty(cellmask)
    cellmask = ones(size(img));
end

imsize = size(img);
masksize = size(cellmask);
    
% Load PSFs
infPSF = imfinfo( psfPath);

psf = zeros( infPSF(1).Height, infPSF(1).Width, length(infPSF));

for I=1:length(infPSF)
    psf(:,:,I)=imread(psfPath,I);
end


psf = psf.^2; % Approximate confocal PSF

%D. Sullivan 3/21/13 initialize the z flip to 0
zflip = 0;

% downsample PDFs and images as specified by user
if exist(imgFile,'file')
    load(imgFile)
else
    fprintf(1,'%s\n','Downsampling PSF and image'); tic;
    psf = ml_downsize(psf,downsample,'average');
    resizeimg = ml_downsize(img,downsample,'average'); toc
    
    if length(size(cellmask)) == 3 & masksize(3) ~= imsize(3)
        [~,ind] = max(squeeze(sum(sum(cellmask,1),2)));
        cellmask = cellmask(:,:,ind);
    end
    
    if length(size(cellmask)) == 3
        resizecellmask = ml_downsize(cellmask,downsample) > 0; 
    else
        resizecellmask = ml_downsize(cellmask, downsample(1:2)) > 0;
        resizecellmask = repmat(resizecellmask, [1,1, size(resizeimg,3)]);
    end
        
    clear dnaim3
    fprintf(1,'%s\n','Deconvolving image'); tic;
    [image,psf2] = deconvblind(resizeimg,psf); toc
%    save(dnafile,'dna_image','dnaPSF2');
end


fprintf(1,'%s\n','Segmenting image'); tic;
sumslices = cumsum(squeeze(sum(sum(image))));
sumslices = sumslices/sumslices(end);
bot_slice=find(sumslices> bot_thresh); %0.05, 0.02
bot_slice=bot_slice(1);
top_slice=find(sumslices< top_thresh); %0.95, 0.98
top_slice=top_slice(end);
%D.Sullivan 6/6/13, these two lines literally do nothing
% bot_slice=max(bot_slice,bot_slice);
% top_slice=min(top_slice,top_slice);

fprintf(1,'Total slices=%i, Bottom slice=%i, Top slice=%i\n',length(sumslices),bot_slice,top_slice);

mask = image > ml_rcthreshold(image(:));
for i = 1:size(mask,3)
    mask(:,:,i) = bwfill(mask(:,:,i), 'holes');
end
mask = repmat(sum(mask,3), [1,1,size(mask,3)]) > 0;

mask = tz_maskimg_3d( mask, resizecellmask );

segimg = region_seg(image, mask, 1000, 0.7, 0, 0.00001);

segimg = ml_findmainobj(segimg);

segimg = tz_maskimg_3d(segimg, resizecellmask);


inds = find(sum(sum(segimg,1),2));
bot_seg = inds(1);
top_seg = inds(end);

bot_slice=max([bot_slice,bot_seg]);
top_slice=min([top_slice,top_seg]);


if bot_slice > bot_seg


% segimg = active3Dsegment(image, bot_slice, top_slice,display,[], 1);

if adherent %false, <param>
    %D. Sullivan  3/21/13 check if image is right side up.
    %since we expect the cell to generally get larger at the bottom for
    %adherent cell lines, if the slope of the areas is positive we should 
    %flip the order

    %get total cell area per slice
    areas = squeeze(sum(sum(segimg)));
    %eliminate ones that are not in the cell
    cellareas = areas(areas~=0);
    %P = coefficients P(1)*X+P(2)
    P = polyfit([1:length(cellareas)],cellareas',1);
    if P(1)>0
        %set zflip flag
        zflip = 1;
        segimg = flipdim(segimg,3);
    end
   [bot_slice,top_slice]=fixadherent(segimg,bot_slice,top_slice);
end
clear image
toc

    
end